@@ -38,15 +38,29 @@
     void* operator new(size_t size) UT_THROW(std::bad_alloc);
     void* operator new[](size_t size) UT_THROW(std::bad_alloc);
 
-    void operator delete(void* mem) UT_NOTHROW;
-    void operator delete[](void* mem) UT_NOTHROW;
     void operator delete(void* mem, const char* file, int line) UT_NOTHROW;
     void operator delete[](void* mem, const char* file, int line) UT_NOTHROW;
+    void operator delete(void* mem) UT_NOTHROW;
+    void operator delete[](void* mem) UT_NOTHROW;
+    void operator delete (void* mem, size_t size) UT_NOTHROW;
+    void operator delete[] (void* mem, size_t size) UT_NOTHROW;
 
 #endif
 
+
+#ifdef __clang__
+ #pragma clang diagnostic push
+ #if __clang_major__ >= 3 && __clang_minor__ >= 6
+  #pragma clang diagnostic ignored "-Wkeyword-macro"
+ #endif
+#endif
+
 #define new new(__FILE__, __LINE__)
 
+#ifdef __clang__
+ #pragma clang diagnostic pop
+#endif
+
 #define CPPUTEST_USE_NEW_MACROS 1
 
 #endif
