@@ -42,15 +42,18 @@ MockSupport& mock(const SimpleString& mockName = "", MockFailureReporter* failur
 class MockSupport
 {
 public:
-    MockSupport();
+    MockSupport(const SimpleString& mockName = "");
     virtual ~MockSupport();
 
     virtual void strictOrder();
     virtual MockExpectedCall& expectOneCall(const SimpleString& functionName);
+    virtual void expectNoCall(const SimpleString& functionName);
     virtual MockExpectedCall& expectNCalls(int amount, const SimpleString& functionName);
     virtual MockActualCall& actualCall(const SimpleString& functionName);
     virtual bool hasReturnValue();
     virtual MockNamedValue returnValue();
+    virtual bool boolReturnValue();
+    virtual bool returnBoolValueOrDefault(bool defaultValue);
     virtual int intReturnValue();
     virtual int returnIntValueOrDefault(int defaultValue);
     virtual unsigned int unsignedIntReturnValue();
@@ -67,14 +70,18 @@ public:
     virtual void* returnPointerValueOrDefault(void * defaultValue);
     virtual const void* returnConstPointerValueOrDefault(const void * defaultValue);
     virtual const void* constPointerReturnValue();
+    virtual void (*returnFunctionPointerValueOrDefault(void (*defaultValue)()))();
+    virtual void (*functionPointerReturnValue())();
 
     bool hasData(const SimpleString& name);
+    void setData(const SimpleString& name, bool value);
     void setData(const SimpleString& name, int value);
     void setData(const SimpleString& name, unsigned int value);
     void setData(const SimpleString& name, const char* value);
     void setData(const SimpleString& name, double value);
     void setData(const SimpleString& name, void* value);
     void setData(const SimpleString& name, const void* value);
+    void setData(const SimpleString& name, void (*value)());
     void setDataObject(const SimpleString& name, const SimpleString& type, void* value);
     MockNamedValue getData(const SimpleString& name);
 
@@ -103,14 +110,15 @@ public:
 
     virtual void setMockFailureStandardReporter(MockFailureReporter* reporter);
     virtual void setActiveReporter(MockFailureReporter* activeReporter);
-    virtual void setDefaultComparatorRepository();
+    virtual void setDefaultComparatorsAndCopiersRepository();
 
     virtual void installComparator(const SimpleString& typeName, MockNamedValueComparator& comparator);
-    virtual void installComparators(const MockNamedValueComparatorRepository& repository);
-    virtual void removeAllComparators();
+    virtual void installCopier(const SimpleString& typeName, MockNamedValueCopier& copier);
+    virtual void installComparatorsAndCopiers(const MockNamedValueComparatorsAndCopiersRepository& repository);
+    virtual void removeAllComparatorsAndCopiers();
 
 protected:
-    MockSupport* clone();
+    MockSupport* clone(const SimpleString& mockName);
     virtual MockCheckedActualCall *createActualFunctionCall();
     virtual void failTest(MockFailure& failure);
     void countCheck();
@@ -123,12 +131,14 @@ private:
     MockFailureReporter *standardReporter_;
     MockFailureReporter defaultReporter_;
     MockExpectedCallsList expectations_;
+    MockExpectedCallsList unExpectations_;
     bool ignoreOtherCalls_;
     bool enabled_;
     MockCheckedActualCall *lastActualFunctionCall_;
     MockExpectedCallComposite compositeCalls_;
-    MockNamedValueComparatorRepository comparatorRepository_;
+    MockNamedValueComparatorsAndCopiersRepository comparatorsAndCopiersRepository_;
     MockNamedValueList data_;
+    const SimpleString mockName_;
 
     bool tracing_;
 
@@ -140,6 +150,13 @@ private:
     MockNamedValue* retrieveDataFromStore(const SimpleString& name);
 
     MockSupport* getMockSupport(MockNamedValueListNode* node);
+
+    bool hasntExpectationWithName(const SimpleString& functionName);
+    bool hasntUnexpectationWithName(const SimpleString& functionName);
+    bool hasCallsOutOfOrder();
+
+    SimpleString appendScopeToName(const SimpleString& functionName);
+
 };
 
 #endif
