@@ -30,7 +30,7 @@
 #include "CppUTest/PlatformSpecificFunctions.h"
 
 CommandLineArguments::CommandLineArguments(int ac, const char** av) :
-    ac_(ac), av_(av), verbose_(false), color_(false), runTestsAsSeperateProcess_(false), listTestGroupNames_(false), listTestGroupAndCaseNames_(false), repeat_(1), groupFilters_(NULL), nameFilters_(NULL), outputType_(OUTPUT_ECLIPSE)
+    ac_(ac), av_(av), verbose_(false), color_(false), runTestsAsSeperateProcess_(false), listTestGroupNames_(false), listTestGroupAndCaseNames_(false), runIgnored_(false), repeat_(1), groupFilters_(NULL), nameFilters_(NULL), outputType_(OUTPUT_ECLIPSE)
 {
 }
 
@@ -53,17 +53,22 @@ bool CommandLineArguments::parse(TestPlugin* plugin)
     bool correctParameters = true;
     for (int i = 1; i < ac_; i++) {
         SimpleString argument = av_[i];
-        
+
         if      (argument == "-v") verbose_ = true;
         else if (argument == "-c") color_ = true;
         else if (argument == "-p") runTestsAsSeperateProcess_ = true;
         else if (argument == "-lg") listTestGroupNames_ = true;
         else if (argument == "-ln") listTestGroupAndCaseNames_ = true;
+        else if (argument == "-ri") runIgnored_ = true;
         else if (argument.startsWith("-r")) SetRepeatCount(ac_, av_, i);
         else if (argument.startsWith("-g")) AddGroupFilter(ac_, av_, i);
         else if (argument.startsWith("-sg")) AddStrictGroupFilter(ac_, av_, i);
+        else if (argument.startsWith("-xg")) AddExcludeGroupFilter(ac_, av_, i);
+        else if (argument.startsWith("-xsg")) AddExcludeStrictGroupFilter(ac_, av_, i);
         else if (argument.startsWith("-n")) AddNameFilter(ac_, av_, i);
         else if (argument.startsWith("-sn")) AddStrictNameFilter(ac_, av_, i);
+        else if (argument.startsWith("-xn")) AddExcludeNameFilter(ac_, av_, i);
+        else if (argument.startsWith("-xsn")) AddExcludeStrictNameFilter(ac_, av_, i);
         else if (argument.startsWith("TEST(")) AddTestToRunBasedOnVerboseOutput(ac_, av_, i, "TEST(");
         else if (argument.startsWith("IGNORE_TEST(")) AddTestToRunBasedOnVerboseOutput(ac_, av_, i, "IGNORE_TEST(");
         else if (argument.startsWith("-o")) correctParameters = SetOutputType(ac_, av_, i);
@@ -80,7 +85,7 @@ bool CommandLineArguments::parse(TestPlugin* plugin)
 
 const char* CommandLineArguments::usage() const
 {
-    return "usage [-v] [-c] [-p] [-lg] [-ln] [-r#] [-g|sg groupName]... [-n|sn testName]... [\"TEST(groupName, testName)\"]... [-o{normal, junit}] [-k packageName]\n";
+    return "usage [-v] [-c] [-p] [-lg] [-ln] [-ri] [-r#] [-g|sg|xg|xsg groupName]... [-n|sn|xn|xsn testName]... [\"TEST(groupName, testName)\"]... [-o{normal, junit, teamcity}] [-k packageName]\n";
 }
 
 bool CommandLineArguments::isVerbose() const
@@ -103,6 +108,11 @@ bool CommandLineArguments::isListingTestGroupAndCaseNames() const
     return listTestGroupAndCaseNames_;
 }
 
+bool CommandLineArguments::isRunIgnored() const
+{
+    return runIgnored_;
+}
+
 bool CommandLineArguments::runTestsInSeperateProcess() const
 {
     return runTestsAsSeperateProcess_;
@@ -161,6 +171,21 @@ void CommandLineArguments::AddStrictGroupFilter(int ac, const char** av, int& i)
     groupFilters_ = groupFilter->add(groupFilters_);
 }
 
+void CommandLineArguments::AddExcludeGroupFilter(int ac, const char** av, int& i)
+{
+    TestFilter* groupFilter = new TestFilter(getParameterField(ac, av, i, "-xg"));
+    groupFilter->invertMatching();
+    groupFilters_ = groupFilter->add(groupFilters_);
+}
+
+void CommandLineArguments::AddExcludeStrictGroupFilter(int ac, const char** av, int& i)
+{
+    TestFilter* groupFilter = new TestFilter(getParameterField(ac, av, i, "-xsg"));
+    groupFilter->strictMatching();
+    groupFilter->invertMatching();
+    groupFilters_ = groupFilter->add(groupFilters_);
+}
+
 void CommandLineArguments::AddNameFilter(int ac, const char** av, int& i)
 {
     TestFilter* nameFilter = new TestFilter(getParameterField(ac, av, i, "-n"));
@@ -174,6 +199,21 @@ void CommandLineArguments::AddStrictNameFilter(int ac, const char** av, int& ind
     nameFilters_= nameFilter->add(nameFilters_);
 }
 
+void CommandLineArguments::AddExcludeNameFilter(int ac, const char** av, int& index)
+{
+    TestFilter* nameFilter = new TestFilter(getParameterField(ac, av, index, "-xn"));
+    nameFilter->invertMatching();
+    nameFilters_= nameFilter->add(nameFilters_);
+}
+
+void CommandLineArguments::AddExcludeStrictNameFilter(int ac, const char** av, int& index)
+{
+    TestFilter* nameFilter = new TestFilter(getParameterField(ac, av, index, "-xsn"));
+    nameFilter->invertMatching();
+    nameFilter->strictMatching();
+    nameFilters_= nameFilter->add(nameFilters_);
+}
+
 void CommandLineArguments::AddTestToRunBasedOnVerboseOutput(int ac, const char** av, int& index, const char* parameterName)
 {
     SimpleString wholename = getParameterField(ac, av, index, parameterName);
@@ -208,6 +248,11 @@ bool CommandLineArguments::SetOutputType(int ac, const char** av, int& i)
         outputType_ = OUTPUT_JUNIT;
         return true;
     }
+    if (outputType == "teamcity") {
+        outputType_ = OUTPUT_TEAMCITY;
+        return true;
+    }
+
     return false;
 }
 
@@ -221,6 +266,11 @@ bool CommandLineArguments::isJUnitOutput() const
     return outputType_ == OUTPUT_JUNIT;
 }
 
+bool CommandLineArguments::isTeamCityOutput() const
+{
+    return outputType_ == OUTPUT_TEAMCITY;
+}
+
 const SimpleString& CommandLineArguments::getPackageName() const
 {
     return packageName_;
