@@ -45,19 +45,19 @@ void SimpleString::setStringAllocator(TestMemoryAllocator* allocator)
 }
 
 /* Avoid using the memory leak detector INSIDE SimpleString as its used inside the detector */
-char* SimpleString::allocStringBuffer(size_t _size)
+char* SimpleString::allocStringBuffer(size_t _size, const char* file, int line)
 {
-    return getStringAllocator()->alloc_memory(_size, __FILE__, __LINE__);
+    return getStringAllocator()->alloc_memory(_size, file, line);
 }
 
-void SimpleString::deallocStringBuffer(char* str)
+void SimpleString::deallocStringBuffer(char* str, const char* file, int line)
 {
-    getStringAllocator()->free_memory(str, __FILE__, __LINE__);
+    getStringAllocator()->free_memory(str, file, line);
 }
 
 char* SimpleString::getEmptyString() const
 {
-    char* empty = allocStringBuffer(1);
+    char* empty = allocStringBuffer(1, __FILE__, __LINE__);
     empty[0] = '\0';
     return empty;
 }
@@ -152,7 +152,7 @@ SimpleString::SimpleString(const char *other, size_t repeatCount)
 {
     size_t otherStringLength = StrLen(other);
     size_t len = otherStringLength * repeatCount + 1;
-    buffer_ = allocStringBuffer(len);
+    buffer_ = allocStringBuffer(len, __FILE__, __LINE__);
     char* next = buffer_;
     for (size_t i = 0; i < repeatCount; i++) {
         StrNCpy(next, other, otherStringLength + 1);
@@ -169,7 +169,7 @@ SimpleString::SimpleString(const SimpleString& other)
 SimpleString& SimpleString::operator=(const SimpleString& other)
 {
     if (this != &other) {
-        deallocStringBuffer(buffer_);
+        deallocStringBuffer(buffer_, __FILE__, __LINE__);
         buffer_ = copyToNewBuffer(other.buffer_);
     }
     return *this;
@@ -224,8 +224,7 @@ void SimpleString::split(const SimpleString& delimiter, SimpleStringCollection&
     for (size_t i = 0; i < num; ++i) {
         prev = str;
         str = StrStr(str, delimiter.buffer_) + 1;
-        size_t len = (size_t) (str - prev) + 1;
-        col[i].buffer_ = copyToNewBuffer(prev, len);
+        col[i] = SimpleString(prev).subString(0, size_t (str - prev));
     }
     if (extraEndToken) {
         col[num] = str;
@@ -250,7 +249,7 @@ void SimpleString::replace(const char* to, const char* with)
     size_t newsize = len + (withlen * c) - (tolen * c) + 1;
 
     if (newsize > 1) {
-        char* newbuf = allocStringBuffer(newsize);
+        char* newbuf = allocStringBuffer(newsize, __FILE__, __LINE__);
         for (size_t i = 0, j = 0; i < len;) {
             if (StrNCmp(&buffer_[i], to, tolen) == 0) {
                 StrNCpy(&newbuf[j], with, withlen + 1);
@@ -263,13 +262,13 @@ void SimpleString::replace(const char* to, const char* with)
                 i++;
             }
         }
-        deallocStringBuffer(buffer_);
+        deallocStringBuffer(buffer_, __FILE__, __LINE__);
         buffer_ = newbuf;
         buffer_[newsize - 1] = '\0';
     }
     else {
+        deallocStringBuffer(buffer_, __FILE__, __LINE__);
         buffer_ = getEmptyString();
-        buffer_[0] = '\0';
     }
 }
 
@@ -302,7 +301,7 @@ bool SimpleString::isEmpty() const
 
 SimpleString::~SimpleString()
 {
-    deallocStringBuffer(buffer_);
+    deallocStringBuffer(buffer_, __FILE__, __LINE__);
 }
 
 bool operator==(const SimpleString& left, const SimpleString& right)
@@ -321,7 +320,7 @@ bool operator!=(const SimpleString& left, const SimpleString& right)
     return !(left == right);
 }
 
-SimpleString SimpleString::operator+(const SimpleString& rhs)
+SimpleString SimpleString::operator+(const SimpleString& rhs) const
 {
     SimpleString t(buffer_);
     t += rhs.buffer_;
@@ -340,7 +339,7 @@ SimpleString& SimpleString::operator+=(const char* rhs)
     size_t sizeOfNewString = originalSize + additionalStringSize;
     char* tbuffer = copyToNewBuffer(this->buffer_, sizeOfNewString);
     StrNCpy(tbuffer + originalSize, rhs, additionalStringSize);
-    deallocStringBuffer(this->buffer_);
+    deallocStringBuffer(this->buffer_, __FILE__, __LINE__);
     this->buffer_ = tbuffer;
     return *this;
 }
@@ -370,40 +369,45 @@ SimpleString SimpleString::subString(size_t beginPos, size_t amount) const
     return newString;
 }
 
-char SimpleString::at(int pos) const
+SimpleString SimpleString::subString(size_t beginPos) const
+{
+    return subString(beginPos, npos);
+}
+
+char SimpleString::at(size_t pos) const
 {
     return buffer_[pos];
 }
 
-int SimpleString::find(char ch) const
+size_t SimpleString::find(char ch) const
 {
     return findFrom(0, ch);
 }
 
-int SimpleString::findFrom(size_t starting_position, char ch) const
+size_t SimpleString::findFrom(size_t starting_position, char ch) const
 {
     size_t length = size();
     for (size_t i = starting_position; i < length; i++)
-        if (buffer_[i] == ch) return (int) i;
-    return -1;
+        if (buffer_[i] == ch) return i;
+    return npos;
 }
 
 SimpleString SimpleString::subStringFromTill(char startChar, char lastExcludedChar) const
 {
-    int beginPos = find(startChar);
-    if (beginPos < 0) return "";
+    size_t beginPos = find(startChar);
+    if (beginPos == npos) return "";
 
-    int endPos = findFrom((size_t)beginPos, lastExcludedChar);
-    if (endPos == -1) return subString((size_t)beginPos, size());
+    size_t endPos = findFrom(beginPos, lastExcludedChar);
+    if (endPos == npos) return subString(beginPos);
 
-    return subString((size_t)beginPos, (size_t) (endPos - beginPos));
+    return subString(beginPos, endPos - beginPos);
 }
 
 char* SimpleString::copyToNewBuffer(const char* bufferToCopy, size_t bufferSize)
 {
     if(bufferSize == 0) bufferSize = StrLen(bufferToCopy) + 1;
 
-    char* newBuffer = allocStringBuffer(bufferSize);
+    char* newBuffer = allocStringBuffer(bufferSize, __FILE__, __LINE__);
     StrNCpy(newBuffer, bufferToCopy, bufferSize);
     newBuffer[bufferSize-1] = '\0';
     return newBuffer;
@@ -464,16 +468,65 @@ SimpleString StringFrom(const void* value)
     return SimpleString("0x") + HexStringFrom(value);
 }
 
+SimpleString StringFrom(void (*value)())
+{
+    return SimpleString("0x") + HexStringFrom(value);
+}
+
 SimpleString HexStringFrom(long value)
 {
     return StringFromFormat("%lx", value);
 }
 
+SimpleString HexStringFrom(signed char value)
+{
+    SimpleString result = StringFromFormat("%x", value);
+    if(value < 0) {
+        size_t size = result.size();
+        result = result.subString(size-(CPPUTEST_CHAR_BIT/4));
+    }
+    return result;
+}
+
 SimpleString HexStringFrom(unsigned long value)
 {
     return StringFromFormat("%lx", value);
 }
 
+#ifdef CPPUTEST_USE_LONG_LONG
+
+SimpleString StringFrom(cpputest_longlong value)
+{
+    return StringFromFormat("%lld", value);
+}
+
+SimpleString StringFrom(cpputest_ulonglong value)
+{
+    return StringFromFormat("%llu (0x%llx)", value, value);
+}
+
+SimpleString HexStringFrom(cpputest_longlong value)
+{
+    return StringFromFormat("%llx", value);
+}
+
+SimpleString HexStringFrom(cpputest_ulonglong value)
+{
+    return StringFromFormat("%llx", value);
+}
+
+SimpleString HexStringFrom(const void* value)
+{
+    return HexStringFrom((cpputest_ulonglong) value);
+}
+
+SimpleString HexStringFrom(void (*value)())
+{
+    return HexStringFrom((cpputest_ulonglong) value);
+}
+
+#else   /* CPPUTEST_USE_LONG_LONG */
+
 static long convertPointerToLongValue(const void* value)
 {
     /*
@@ -485,14 +538,57 @@ static long convertPointerToLongValue(const void* value)
     return *long_value;
 }
 
+static long convertFunctionPointerToLongValue(void (*value)())
+{
+    /*
+     * This way of converting also can convert a 64bit pointer in a 32bit integer by truncating.
+     * This isn't the right way to convert pointers values and need to change by implementing a
+     * proper portable way to convert pointers to strings.
+     */
+    long* long_value = (long*) &value;
+    return *long_value;
+}
+
+SimpleString StringFrom(cpputest_longlong)
+{
+    return "<longlong_unsupported>";
+}
+
+SimpleString StringFrom(cpputest_ulonglong)
+{
+    return "<ulonglong_unsupported>";
+}
+
+SimpleString HexStringFrom(cpputest_longlong)
+{
+    return "<longlong_unsupported>";
+}
+
+SimpleString HexStringFrom(cpputest_ulonglong)
+{
+    return "<ulonglong_unsupported>";
+}
+
 SimpleString HexStringFrom(const void* value)
 {
     return StringFromFormat("%lx", convertPointerToLongValue(value));
 }
 
+SimpleString HexStringFrom(void (*value)())
+{
+    return StringFromFormat("%lx", convertFunctionPointerToLongValue(value));
+}
+
+#endif  /* CPPUTEST_USE_LONG_LONG */
+
 SimpleString StringFrom(double value, int precision)
 {
-    return StringFromFormat("%.*g", precision, value);
+    if (PlatformSpecificIsNan(value))
+        return "Nan - Not a number";
+    else if (PlatformSpecificIsInf(value))
+        return "Inf - Infinity";
+    else
+        return StringFromFormat("%.*g", precision, value);
 }
 
 SimpleString StringFrom(char value)
@@ -559,11 +655,11 @@ SimpleString VStringFromFormat(const char* format, va_list args)
     }
     else {
         size_t newBufferSize = size + 1;
-        char* newBuffer = SimpleString::allocStringBuffer(newBufferSize);
+        char* newBuffer = SimpleString::allocStringBuffer(newBufferSize, __FILE__, __LINE__);
         PlatformSpecificVSNprintf(newBuffer, newBufferSize, format, argsCopy);
         resultString = SimpleString(newBuffer);
 
-        SimpleString::deallocStringBuffer(newBuffer);
+        SimpleString::deallocStringBuffer(newBuffer, __FILE__, __LINE__);
     }
     va_end(argsCopy);
     return resultString;
@@ -586,6 +682,68 @@ SimpleString StringFromBinaryOrNull(const unsigned char* value, size_t size)
     return (value) ? StringFromBinary(value, size) : "(null)";
 }
 
+SimpleString StringFromBinaryWithSize(const unsigned char* value, size_t size)
+{
+    SimpleString result = StringFromFormat("Size = %u | HexContents = ", (unsigned) size);
+    size_t displayedSize = ((size > 128) ? 128 : size);
+    result += StringFromBinaryOrNull(value, displayedSize);
+    if (size > displayedSize)
+    {
+        result += " ...";
+    }
+    return result;
+}
+
+SimpleString StringFromBinaryWithSizeOrNull(const unsigned char* value, size_t size)
+{
+    return (value) ? StringFromBinaryWithSize(value, size) : "(null)";
+}
+
+SimpleString StringFromMaskedBits(unsigned long value, unsigned long mask, size_t byteCount)
+{
+    SimpleString result;
+    size_t bitCount = (byteCount > sizeof(unsigned long)) ? (sizeof(unsigned long) * CPPUTEST_CHAR_BIT) : (byteCount * CPPUTEST_CHAR_BIT);
+    const unsigned long msbMask = (((unsigned long) 1) << (bitCount - 1));
+
+    for (size_t i = 0; i < bitCount; i++) {
+        if (mask & msbMask) {
+            result += (value & msbMask) ? "1" : "0";
+        }
+        else {
+            result += "x";
+        }
+
+        if (((i % 8) == 7) && (i != (bitCount - 1))) {
+            result += " ";
+        }
+
+        value <<= 1;
+        mask <<= 1;
+    }
+
+    return result;
+}
+
+SimpleString StringFromOrdinalNumber(unsigned int number)
+{
+    unsigned int onesDigit = number % 10;
+
+    const char* suffix;
+    if (number >= 11 && number <= 13) {
+        suffix = "th";
+    } else if (3 == onesDigit) {
+        suffix = "rd";
+    } else if (2 == onesDigit) {
+        suffix = "nd";
+    } else if (1 == onesDigit) {
+        suffix = "st";
+    } else {
+        suffix = "th";
+    }
+
+    return StringFromFormat("%u%s", number, suffix);
+}
+
 SimpleStringCollection::SimpleStringCollection()
 {
     collection_ = 0;
