@@ -28,16 +28,16 @@
 #include "CppUTest/CppUTestConfig.h"
 #include "CppUTest/TestFilter.h"
 
-TestFilter::TestFilter() : strictMatching_(false), next_(NULL)
+TestFilter::TestFilter() : strictMatching_(false), invertMatching_(false), next_(NULL)
 {
 }
 
-TestFilter::TestFilter(const SimpleString& filter) : strictMatching_(false), next_(NULL)
+TestFilter::TestFilter(const SimpleString& filter) : strictMatching_(false), invertMatching_(false), next_(NULL)
 {
     filter_ = filter;
 }
 
-TestFilter::TestFilter(const char* filter) : strictMatching_(false), next_(NULL)
+TestFilter::TestFilter(const char* filter) : strictMatching_(false), invertMatching_(false), next_(NULL)
 {
     filter_ = filter;
 }
@@ -58,16 +58,28 @@ void TestFilter::strictMatching()
     strictMatching_ = true;
 }
 
+void TestFilter::invertMatching()
+{
+    invertMatching_ = true;
+}
+
 bool TestFilter::match(const SimpleString& name) const
 {
+    bool matches = false;
+
     if(strictMatching_)
-        return name == filter_;
-    return name.contains(filter_);
+        matches = name == filter_;
+    else
+        matches = name.contains(filter_);
+
+    return invertMatching_ ? !matches : matches;
 }
 
 bool TestFilter::operator==(const TestFilter& filter) const
 {
-    return (filter_ == filter.filter_ && strictMatching_ == filter.strictMatching_);
+    return (filter_ == filter.filter_ &&
+            strictMatching_ == filter.strictMatching_ &&
+            invertMatching_ == filter.invertMatching_);
 }
 
 bool TestFilter::operator!=(const TestFilter& filter) const
@@ -78,8 +90,13 @@ bool TestFilter::operator!=(const TestFilter& filter) const
 SimpleString TestFilter::asString() const
 {
     SimpleString textFilter =  StringFromFormat("TestFilter: \"%s\"", filter_.asCharString());
-    if (strictMatching_)
+    if (strictMatching_ && invertMatching_)
+        textFilter += " with strict, invert matching";
+    else if (strictMatching_)
         textFilter += " with strict matching";
+    else if (invertMatching_)
+        textFilter += " with invert matching";
+
     return textFilter;
 }
 
