@@ -202,3 +202,120 @@ TestMemoryAllocator* NullUnknownAllocator::defaultAllocator()
     static NullUnknownAllocator allocator;
     return &allocator;
 }
+
+class LocationToFailAllocNode
+{
+  public:
+    int allocNumberToFail_;
+    int actualAllocNumber_;
+    const char* file_;
+    int line_;
+    LocationToFailAllocNode* next_;
+
+    void failAtAllocNumber(int number, LocationToFailAllocNode* next)
+    {
+      init(next);
+      allocNumberToFail_ = number;
+    }
+
+    void failNthAllocAt(int allocationNumber, const char* file, int line, LocationToFailAllocNode* next)
+    {
+      init(next);
+      allocNumberToFail_ = allocationNumber;
+      file_ = file;
+      line_ = line;
+    }
+
+    bool shouldFail(int allocationNumber, const char* file, int line)
+    {
+      if (file_ && SimpleString::StrCmp(file, file_) == 0 && line == line_) {
+        actualAllocNumber_++;
+        return actualAllocNumber_ == allocNumberToFail_;
+      }
+      if (allocationNumber == allocNumberToFail_)
+        return true;
+      return false;
+    }
+
+  private:
+    void init(LocationToFailAllocNode* next = NULL)
+    {
+      allocNumberToFail_ = 0;
+      actualAllocNumber_ = 0;
+      file_ = NULL;
+      line_ = 0;
+      next_ = next;
+    }
+
+};
+
+FailableMemoryAllocator::FailableMemoryAllocator(const char* name_str, const char* alloc_name_str, const char* free_name_str)
+: TestMemoryAllocator(name_str, alloc_name_str, free_name_str), head_(NULL), currentAllocNumber_(0)
+{
+}
+
+void FailableMemoryAllocator::failAllocNumber(int number)
+{
+    LocationToFailAllocNode* newNode = (LocationToFailAllocNode*) (void*) allocMemoryLeakNode(sizeof(LocationToFailAllocNode));
+    newNode->failAtAllocNumber(number, head_);
+    head_ = newNode;
+}
+
+void FailableMemoryAllocator::failNthAllocAt(int allocationNumber, const char* file, int line)
+{
+    LocationToFailAllocNode* newNode = (LocationToFailAllocNode*) (void*) allocMemoryLeakNode(sizeof(LocationToFailAllocNode));
+    newNode->failNthAllocAt(allocationNumber, file, line, head_);
+    head_ = newNode;
+}
+
+char* FailableMemoryAllocator::alloc_memory(size_t size, const char* file, int line)
+{
+    currentAllocNumber_++;
+    LocationToFailAllocNode* current = head_;
+    LocationToFailAllocNode* previous = NULL;
+
+    while (current) {
+      if (current->shouldFail(currentAllocNumber_, file, line)) {
+        if (previous) previous->next_ = current->next_;
+        else head_ = current->next_;
+
+        free_memory((char*) current, __FILE__, __LINE__);
+        return NULL;
+      }
+      previous = current;
+      current = current->next_;
+    }
+    return TestMemoryAllocator::alloc_memory(size, file, line);
+}
+
+char* FailableMemoryAllocator::allocMemoryLeakNode(size_t size)
+{
+    return (char*)PlatformSpecificMalloc(size);
+}
+
+void FailableMemoryAllocator::checkAllFailedAllocsWereDone()
+{
+  if (head_) {
+    UtestShell* currentTest = UtestShell::getCurrent();
+    SimpleString failText;
+    if (head_->file_)
+      failText = StringFromFormat("Expected failing alloc at %s:%d was never done", head_->file_, head_->line_);
+    else
+      failText = StringFromFormat("Expected allocation number %d was never done", head_->allocNumberToFail_);
+
+    currentTest->failWith(FailFailure(currentTest, currentTest->getName().asCharString(),
+    currentTest->getLineNumber(), failText), TestTerminatorWithoutExceptions());
+  }
+}
+
+void FailableMemoryAllocator::clearFailedAllocs()
+{
+  LocationToFailAllocNode* current = head_;
+  while (current) {
+    head_ = current->next_;
+    free_memory((char*) current, __FILE__, __LINE__);
+    current = head_;
+  }
+  currentAllocNumber_ = 0;
+}
+
