@@ -73,6 +73,14 @@ MockExpectedCall& MockCheckedExpectedCall::withName(const SimpleString& name)
     return *this;
 }
 
+MockExpectedCall& MockCheckedExpectedCall::withBoolParameter(const SimpleString& name, bool value)
+{
+    MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
+    inputParameters_->add(newParameter);
+    newParameter->setValue(value);
+    return *this;
+}
+
 MockExpectedCall& MockCheckedExpectedCall::withUnsignedIntParameter(const SimpleString& name, unsigned int value)
 {
     MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
@@ -137,6 +145,22 @@ MockExpectedCall& MockCheckedExpectedCall::withConstPointerParameter(const Simpl
     return *this;
 }
 
+MockExpectedCall& MockCheckedExpectedCall::withFunctionPointerParameter(const SimpleString& name, void (*value)())
+{
+    MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
+    inputParameters_->add(newParameter);
+    newParameter->setValue(value);
+    return *this;
+}
+
+MockExpectedCall& MockCheckedExpectedCall::withMemoryBufferParameter(const SimpleString& name, const unsigned char* value, size_t size)
+{
+    MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
+    inputParameters_->add(newParameter);
+    newParameter->setMemoryBuffer(value, size);
+    return *this;
+}
+
 MockExpectedCall& MockCheckedExpectedCall::withParameterOfType(const SimpleString& type, const SimpleString& name, const void* value)
 {
     MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
@@ -154,6 +178,14 @@ MockExpectedCall& MockCheckedExpectedCall::withOutputParameterReturning(const Si
     return *this;
 }
 
+MockExpectedCall& MockCheckedExpectedCall::withOutputParameterOfTypeReturning(const SimpleString& type, const SimpleString& name, const void* value)
+{
+    MockNamedValue* newParameter = new MockExpectedFunctionParameter(name);
+    outputParameters_->add(newParameter);
+    newParameter->setObjectPointer(type, value);
+    return *this;
+}
+
 SimpleString MockCheckedExpectedCall::getInputParameterType(const SimpleString& name)
 {
     MockNamedValue * p = inputParameters_->getValueByName(name);
@@ -166,6 +198,12 @@ bool MockCheckedExpectedCall::hasInputParameterWithName(const SimpleString& name
     return p != NULL;
 }
 
+bool MockCheckedExpectedCall::hasOutputParameterWithName(const SimpleString& name)
+{
+    MockNamedValue * p = outputParameters_->getValueByName(name);
+    return p != NULL;
+}
+
 MockNamedValue MockCheckedExpectedCall::getInputParameter(const SimpleString& name)
 {
     MockNamedValue * p = inputParameters_->getValueByName(name);
@@ -279,7 +317,7 @@ bool MockCheckedExpectedCall::hasInputParameter(const MockNamedValue& parameter)
 bool MockCheckedExpectedCall::hasOutputParameter(const MockNamedValue& parameter)
 {
     MockNamedValue * p = outputParameters_->getValueByName(parameter.getName());
-    return (p) ? true : ignoreOtherParameters_;
+    return (p) ? p->compatibleForCopying(parameter) : ignoreOtherParameters_;
 }
 
 SimpleString MockCheckedExpectedCall::callToString()
@@ -306,6 +344,11 @@ SimpleString MockCheckedExpectedCall::callToString()
         if (p->next()) str += ", ";
     }
 
+    if (inputParameters_->begin() && outputParameters_->begin())
+    {
+        str += ", ";
+    }
+
     for (p = outputParameters_->begin(); p; p = p->next()) {
         str += StringFromFormat("%s %s: <output>", p->getType().asCharString(), p->getName().asCharString());
         if (p->next()) str += ", ";
@@ -341,7 +384,7 @@ bool MockCheckedExpectedCall::relatesTo(const SimpleString& functionName)
     return functionName == getName();
 }
 
-bool MockCheckedExpectedCall::relatesToObject(void*objectPtr) const
+bool MockCheckedExpectedCall::relatesToObject(const void* objectPtr) const
 {
     return objectPtr_ == objectPtr;
 }
@@ -366,6 +409,13 @@ bool MockCheckedExpectedCall::MockExpectedFunctionParameter::isFulfilled() const
     return fulfilled_;
 }
 
+MockExpectedCall& MockCheckedExpectedCall::andReturnValue(bool value)
+{
+    returnValue_.setName("returnValue");
+    returnValue_.setValue(value);
+    return *this;
+}
+
 MockExpectedCall& MockCheckedExpectedCall::andReturnValue(unsigned int value)
 {
     returnValue_.setName("returnValue");
@@ -422,6 +472,13 @@ MockExpectedCall& MockCheckedExpectedCall::andReturnValue(const void* value)
     return *this;
 }
 
+MockExpectedCall& MockCheckedExpectedCall::andReturnValue(void (*value)())
+{
+    returnValue_.setName("returnValue");
+    returnValue_.setValue(value);
+    return *this;
+}
+
 MockExpectedCall& MockCheckedExpectedCall::onObject(void* objectPtr)
 {
     wasPassedToObject_ = false;
@@ -494,6 +551,13 @@ MockExpectedCall& MockExpectedCallComposite::withCallOrder(int)
     return *this; // LCOV_EXCL_LINE
 }
 
+MockExpectedCall& MockExpectedCallComposite::withBoolParameter(const SimpleString& name, bool value)
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.withParameter(name, value);
+    return *this;
+}
+
 MockExpectedCall& MockExpectedCallComposite::withUnsignedIntParameter(const SimpleString& name, unsigned int value)
 {
     for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
@@ -551,6 +615,20 @@ MockExpectedCall& MockExpectedCallComposite::withConstPointerParameter(const Sim
     return *this;
 }
 
+MockExpectedCall& MockExpectedCallComposite::withFunctionPointerParameter(const SimpleString& name, void (*value)())
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.withParameter(name, value);
+    return *this;
+}
+
+MockExpectedCall& MockExpectedCallComposite::withMemoryBufferParameter(const SimpleString& name, const unsigned char* value, size_t size)
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.withParameter(name, value, size);
+    return *this;
+}
+
 MockExpectedCall& MockExpectedCallComposite::withParameterOfType(const SimpleString& typeName, const SimpleString& name, const void* value)
 {
     for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
@@ -565,6 +643,13 @@ MockExpectedCall& MockExpectedCallComposite::withOutputParameterReturning(const
     return *this;
 }
 
+MockExpectedCall& MockExpectedCallComposite::withOutputParameterOfTypeReturning(const SimpleString& typeName, const SimpleString& name, const void* value)
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.withOutputParameterOfTypeReturning(typeName, name, value);
+    return *this;
+}
+
 MockExpectedCall& MockExpectedCallComposite::ignoreOtherParameters()
 {
     for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
@@ -572,6 +657,13 @@ MockExpectedCall& MockExpectedCallComposite::ignoreOtherParameters()
     return *this;
 }
 
+MockExpectedCall& MockExpectedCallComposite::andReturnValue(bool value)
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.andReturnValue(value);
+    return *this;
+}
+
 MockExpectedCall& MockExpectedCallComposite::andReturnValue(unsigned int value)
 {
     for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
@@ -628,6 +720,13 @@ MockExpectedCall& MockExpectedCallComposite::andReturnValue(const void* value)
     return *this;
 }
 
+MockExpectedCall& MockExpectedCallComposite::andReturnValue(void (*value)())
+{
+    for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
+        node->call_.andReturnValue(value);
+    return *this;
+}
+
 MockExpectedCall& MockExpectedCallComposite::onObject(void* object)
 {
     for (MockExpectedCallCompositeNode* node = head_; node != NULL; node = node->next_)
