@@ -39,12 +39,12 @@ MockSupport& mock(const SimpleString& mockName, MockFailureReporter* failureRepo
 {
     MockSupport& mock_support = (mockName != "") ? *global_mock.getMockSupportScope(mockName) : global_mock;
     mock_support.setActiveReporter(failureReporterForThisCall);
-    mock_support.setDefaultComparatorRepository();
+    mock_support.setDefaultComparatorsAndCopiersRepository();
     return mock_support;
 }
 
-MockSupport::MockSupport()
-    : callOrder_(0), expectedCallOrder_(0), strictOrdering_(false), standardReporter_(&defaultReporter_), ignoreOtherCalls_(false), enabled_(true), lastActualFunctionCall_(NULL), tracing_(false)
+MockSupport::MockSupport(const SimpleString& mockName)
+    : callOrder_(0), expectedCallOrder_(0), strictOrdering_(false), standardReporter_(&defaultReporter_), ignoreOtherCalls_(false), enabled_(true), lastActualFunctionCall_(NULL), mockName_(mockName), tracing_(false)
 {
     setActiveReporter(NULL);
 }
@@ -74,32 +74,40 @@ void MockSupport::setActiveReporter(MockFailureReporter* reporter)
     activeReporter_ = (reporter) ? reporter : standardReporter_;
 }
 
-void MockSupport::setDefaultComparatorRepository()
+void MockSupport::setDefaultComparatorsAndCopiersRepository()
 {
-    MockNamedValue::setDefaultComparatorRepository(&comparatorRepository_);
+    MockNamedValue::setDefaultComparatorsAndCopiersRepository(&comparatorsAndCopiersRepository_);
 }
 
 void MockSupport::installComparator(const SimpleString& typeName, MockNamedValueComparator& comparator)
 {
-    comparatorRepository_.installComparator(typeName, comparator);
+    comparatorsAndCopiersRepository_.installComparator(typeName, comparator);
 
     for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
         if (getMockSupport(p)) getMockSupport(p)->installComparator(typeName, comparator);
 }
 
-void MockSupport::installComparators(const MockNamedValueComparatorRepository& repository)
+void MockSupport::installCopier(const SimpleString& typeName, MockNamedValueCopier& copier)
 {
-    comparatorRepository_.installComparators(repository);
+    comparatorsAndCopiersRepository_.installCopier(typeName, copier);
 
     for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
-        if (getMockSupport(p)) getMockSupport(p)->installComparators(repository);
+        if (getMockSupport(p)) getMockSupport(p)->installCopier(typeName, copier);
 }
 
-void MockSupport::removeAllComparators()
+void MockSupport::installComparatorsAndCopiers(const MockNamedValueComparatorsAndCopiersRepository& repository)
 {
-    comparatorRepository_.clear();
+    comparatorsAndCopiersRepository_.installComparatorsAndCopiers(repository);
+
     for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
-        if (getMockSupport(p)) getMockSupport(p)->removeAllComparators();
+        if (getMockSupport(p)) getMockSupport(p)->installComparatorsAndCopiers(repository);
+}
+
+void MockSupport::removeAllComparatorsAndCopiers()
+{
+    comparatorsAndCopiersRepository_.clear();
+    for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
+        if (getMockSupport(p)) getMockSupport(p)->removeAllComparatorsAndCopiers();
 }
 
 void MockSupport::clear()
@@ -111,6 +119,7 @@ void MockSupport::clear()
     MockActualCallTrace::instance().clear();
 
     expectations_.deleteAllExpectationsAndClearList();
+    unExpectations_.deleteAllExpectationsAndClearList();
     compositeCalls_.clear();
     ignoreOtherCalls_ = false;
     enabled_ = true;
@@ -133,6 +142,12 @@ void MockSupport::strictOrder()
     strictOrdering_ = true;
 }
 
+SimpleString MockSupport::appendScopeToName(const SimpleString& functionName)
+{
+    if (mockName_.isEmpty()) return functionName;
+    return mockName_ + "::" + functionName;
+}
+
 MockExpectedCall& MockSupport::expectOneCall(const SimpleString& functionName)
 {
     if (!enabled_) return MockIgnoredExpectedCall::instance();
@@ -140,13 +155,24 @@ MockExpectedCall& MockSupport::expectOneCall(const SimpleString& functionName)
     countCheck();
 
     MockCheckedExpectedCall* call = new MockCheckedExpectedCall;
-    call->withName(functionName);
+    call->withName(appendScopeToName(functionName));
     if (strictOrdering_)
         call->withCallOrder(++expectedCallOrder_);
     expectations_.addExpectedCall(call);
     return *call;
 }
 
+void MockSupport::expectNoCall(const SimpleString& functionName)
+{
+    if (!enabled_) return;
+
+    countCheck();
+
+    MockCheckedExpectedCall* call = new MockCheckedExpectedCall;
+    call->withName(appendScopeToName(functionName));
+    unExpectations_.addExpectedCall(call);
+}
+
 MockExpectedCall& MockSupport::expectNCalls(int amount, const SimpleString& functionName)
 {
     compositeCalls_.clear();
@@ -162,8 +188,20 @@ MockCheckedActualCall* MockSupport::createActualFunctionCall()
     return lastActualFunctionCall_;
 }
 
+bool MockSupport::hasntExpectationWithName(const SimpleString& functionName)
+{
+    return !expectations_.hasExpectationWithName(functionName) && ignoreOtherCalls_;
+}
+
+bool MockSupport::hasntUnexpectationWithName(const SimpleString& functionName)
+{
+    return !unExpectations_.hasExpectationWithName(functionName);
+}
+
 MockActualCall& MockSupport::actualCall(const SimpleString& functionName)
 {
+    const SimpleString scopeFuntionName = appendScopeToName(functionName);
+
     if (lastActualFunctionCall_) {
         lastActualFunctionCall_->checkExpectations();
         delete lastActualFunctionCall_;
@@ -171,15 +209,15 @@ MockActualCall& MockSupport::actualCall(const SimpleString& functionName)
     }
 
     if (!enabled_) return MockIgnoredActualCall::instance();
-    if (tracing_) return MockActualCallTrace::instance().withName(functionName);
+    if (tracing_) return MockActualCallTrace::instance().withName(scopeFuntionName);
 
 
-    if (!expectations_.hasExpectationWithName(functionName) && ignoreOtherCalls_) {
+    if (hasntUnexpectationWithName(scopeFuntionName) && hasntExpectationWithName(scopeFuntionName)) {
         return MockIgnoredActualCall::instance();
     }
 
     MockCheckedActualCall* call = createActualFunctionCall();
-    call->withName(functionName);
+    call->withName(scopeFuntionName);
     return *call;
 }
 
@@ -222,7 +260,7 @@ const char* MockSupport::getTraceOutput()
 
 bool MockSupport::expectedCallsLeft()
 {
-    int callsLeft = expectations_.hasUnfullfilledExpectations();
+    int callsLeft = expectations_.hasUnfulfilledExpectations();
 
     for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
         if (getMockSupport(p)) callsLeft += getMockSupport(p)->expectedCallsLeft();
@@ -290,6 +328,20 @@ void MockSupport::checkExpectationsOfLastCall()
             getMockSupport(p)->lastActualFunctionCall_->checkExpectations();
 }
 
+bool MockSupport::hasCallsOutOfOrder()
+{
+    if (expectations_.hasCallsOutOfOrder())
+    {
+        return true;
+    }
+    for (MockNamedValueListNode* p = data_.begin(); p; p = p->next())
+        if (getMockSupport(p) && getMockSupport(p)->hasCallsOutOfOrder())
+        {
+            return true;
+        }
+    return false;
+}
+
 void MockSupport::checkExpectations()
 {
     checkExpectationsOfLastCall();
@@ -297,10 +349,11 @@ void MockSupport::checkExpectations()
     if (wasLastCallFulfilled() && expectedCallsLeft())
         failTestWithUnexpectedCalls();
 
-    if (expectations_.hasCallsOutOfOrder())
+    if (hasCallsOutOfOrder())
         failTestWithOutOfOrderCalls();
 }
 
+
 bool MockSupport::hasData(const SimpleString& name)
 {
     return data_.getValueByName(name) != NULL;
@@ -316,6 +369,12 @@ MockNamedValue* MockSupport::retrieveDataFromStore(const SimpleString& name)
     return newData;
 }
 
+void MockSupport::setData(const SimpleString& name, bool value)
+{
+    MockNamedValue* newData = retrieveDataFromStore(name);
+    newData->setValue(value);
+}
+
 void MockSupport::setData(const SimpleString& name, unsigned int value)
 {
     MockNamedValue* newData = retrieveDataFromStore(name);
@@ -352,6 +411,12 @@ void MockSupport::setData(const SimpleString& name, const void* value)
     newData->setValue(value);
 }
 
+void MockSupport::setData(const SimpleString& name, void (*value)())
+{
+    MockNamedValue* newData = retrieveDataFromStore(name);
+    newData->setValue(value);
+}
+
 void MockSupport::setDataObject(const SimpleString& name, const SimpleString& type, void* value)
 {
     MockNamedValue* newData = retrieveDataFromStore(name);
@@ -366,9 +431,9 @@ MockNamedValue MockSupport::getData(const SimpleString& name)
     return *value;
 }
 
-MockSupport* MockSupport::clone()
+MockSupport* MockSupport::clone(const SimpleString& mockName)
 {
-    MockSupport* newMock = new MockSupport;
+    MockSupport* newMock = new MockSupport(mockName);
     newMock->setMockFailureStandardReporter(standardReporter_);
     if (ignoreOtherCalls_) newMock->ignoreOtherCalls();
 
@@ -377,7 +442,7 @@ MockSupport* MockSupport::clone()
     if (strictOrdering_) newMock->strictOrder();
 
     newMock->tracing(tracing_);
-    newMock->installComparators(comparatorRepository_);
+    newMock->installComparatorsAndCopiers(comparatorsAndCopiersRepository_);
     return newMock;
 }
 
@@ -391,7 +456,7 @@ MockSupport* MockSupport::getMockSupportScope(const SimpleString& name)
         return (MockSupport*) getData(mockingSupportName).getObjectPointer();
     }
 
-    MockSupport *newMock = clone();
+    MockSupport *newMock = clone(name);
 
     setDataObject(mockingSupportName, "MockSupport", newMock);
     return newMock;
@@ -410,6 +475,11 @@ MockNamedValue MockSupport::returnValue()
     return MockNamedValue("");
 }
 
+bool MockSupport::boolReturnValue()
+{
+    return returnValue().getBoolValue();
+}
+
 unsigned int MockSupport::unsignedIntReturnValue()
 {
     return returnValue().getUnsignedIntValue();
@@ -444,6 +514,14 @@ long int MockSupport::returnLongIntValueOrDefault(long int defaultValue)
     return defaultValue;
 }
 
+bool MockSupport::returnBoolValueOrDefault(bool defaultValue)
+{
+    if (hasReturnValue()) {
+        return boolReturnValue();
+    }
+    return defaultValue;
+}
+
 int MockSupport::returnIntValueOrDefault(int defaultValue)
 {
     if (hasReturnValue()) {
@@ -504,6 +582,14 @@ const void* MockSupport::returnConstPointerValueOrDefault(const void * defaultVa
     return defaultValue;
 }
 
+void (*MockSupport::returnFunctionPointerValueOrDefault(void (*defaultValue)()))()
+{
+    if (hasReturnValue()) {
+        return functionPointerReturnValue();
+    }
+    return defaultValue;
+}
+
 void* MockSupport::pointerReturnValue()
 {
     return returnValue().getPointerValue();
@@ -514,6 +600,11 @@ const void* MockSupport::constPointerReturnValue()
     return returnValue().getConstPointerValue();
 }
 
+void (*MockSupport::functionPointerReturnValue())()
+{
+    return returnValue().getFunctionPointerValue();
+}
+
 bool MockSupport::hasReturnValue()
 {
     if (lastActualFunctionCall_) return lastActualFunctionCall_->hasReturnValue();
