@@ -12,16 +12,16 @@ typedef struct AllocLetTestFreeStruct
 
 AllocLetTestFree AllocLetTestFree_Create(void)
 {
-	size_t count = 1;
-    AllocLetTestFree self = calloc(count, sizeof(AllocLetTestFreeStruct));
+    size_t count = 1;
+    AllocLetTestFree self = (AllocLetTestFree)calloc(count, sizeof(AllocLetTestFreeStruct));
     return self;
 }
 
 void AllocLetTestFree_Destroy(AllocLetTestFree self)
 {
-	void* no_use = self;
-	self = NULL;
-	self = no_use;
+    AllocLetTestFree no_use = self;
+    self = NULL;
+    self = no_use;
 
 }
 #endif
