@@ -131,6 +131,38 @@ TEST(TestHarness_c, checkChar)
     CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
 }
 
+static void _failUnsignedByteMethod()
+{
+    HasTheDestructorBeenCalledChecker checker;
+    CHECK_EQUAL_C_UBYTE(254, 253);
+}
+
+TEST(TestHarness_c, checkUnsignedByte)
+{
+    CHECK_EQUAL_C_UBYTE(254, 254);
+    fixture->setTestFunction(_failUnsignedByteMethod);
+    fixture->runAllTests();
+    fixture->assertPrintContains("expected <254>\n	but was  <253>");
+    fixture->assertPrintContains("arness_c");
+    CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
+}
+
+static void _failSignedByteMethod()
+{
+    HasTheDestructorBeenCalledChecker checker;
+    CHECK_EQUAL_C_SBYTE(-3, -5);
+}
+
+TEST(TestHarness_c, checkSignedByte)
+{
+    CHECK_EQUAL_C_SBYTE(-3, -3);
+    fixture->setTestFunction(_failSignedByteMethod);
+    fixture->runAllTests();
+    fixture->assertPrintContains("expected <-3>\n	but was  <-5>");
+    fixture->assertPrintContains("arness_c");
+    CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
+}
+
 static void _failStringMethod()
 {
     HasTheDestructorBeenCalledChecker checker;
@@ -143,12 +175,44 @@ TEST(TestHarness_c, checkString)
     fixture->setTestFunction(_failStringMethod);
     fixture->runAllTests();
 
-    StringEqualFailure failure(UtestShell::getCurrent(), "file", 1, "Hello", "Hello World");
+    StringEqualFailure failure(UtestShell::getCurrent(), "file", 1, "Hello", "Hello World", "");
     fixture->assertPrintContains(failure.getMessage());
     fixture->assertPrintContains("arness_c");
     CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
 }
 
+static void _failPointerMethod()
+{
+    HasTheDestructorBeenCalledChecker checker;
+    CHECK_EQUAL_C_POINTER(NULL, (void *)0x1);
+}
+
+TEST(TestHarness_c, checkPointer)
+{
+    CHECK_EQUAL_C_POINTER(NULL, NULL);
+    fixture->setTestFunction(_failPointerMethod);
+    fixture->runAllTests();
+    fixture->assertPrintContains("expected <0x0>\n	but was  <0x1>");
+    fixture->assertPrintContains("arness_c");
+    CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
+}
+
+static void _failBitsMethod()
+{
+    HasTheDestructorBeenCalledChecker checker;
+    CHECK_EQUAL_C_BITS(0x0001, (unsigned short)0x0003, 0xFFFF);
+}
+
+TEST(TestHarness_c, checkBits)
+{
+    CHECK_EQUAL_C_POINTER(NULL, NULL);
+    fixture->setTestFunction(_failBitsMethod);
+    fixture->runAllTests();
+    fixture->assertPrintContains("expected <00000000 00000001>\n\tbut was  <00000000 00000011>");
+    fixture->assertPrintContains("arness_c");
+    CHECK(!hasDestructorOfTheDestructorCheckedBeenCalled)
+}
+
 static void _failTextMethod()
 {
     HasTheDestructorBeenCalledChecker checker;
