@@ -30,29 +30,23 @@
 #include "CppUTest/TestTestingFixture.h"
 #include "CppUTest/PlatformSpecificFunctions.h"
 
+// This will cause a crash in VS2010 due to PlatformSpecificFree being uninitialized
+static const SimpleString str1("abc");
+static const SimpleString str2("def");
+static const SimpleString str3(str1 + str2);
+
 TEST_GROUP(UTestPlatformsTest_PlatformSpecificRunTestInASeperateProcess)
 {
     TestTestingFixture fixture;
 };
 
-#if defined(__MINGW32__)
+#ifndef CPPUTEST_HAVE_FORK
 
-TEST(UTestPlatformsTest_PlatformSpecificRunTestInASeperateProcess, MinGwWorks)
+TEST(UTestPlatformsTest_PlatformSpecificRunTestInASeperateProcess, DummyFailsWithMessage)
 {
     fixture.registry_->setRunTestsInSeperateProcess();
     fixture.runAllTests();
-    fixture.assertPrintContains(
-       "-p doesn't work on MinGW as it is lacking fork.");
-}
-
-#elif defined(_MSC_VER)
-
-TEST(UTestPlatformsTest_PlatformSpecificRunTestInASeperateProcess, VisualCppWorks)
-{
-   fixture.registry_->setRunTestsInSeperateProcess();
-   fixture.runAllTests();
-   fixture.assertPrintContains(
-      "-p doesn't work on Visual C++ as it is lacking fork.");
+    fixture.assertPrintContains("-p doesn't work on this platform, as it is lacking fork.\b");
 }
 
 #else
